/**** A P P L I C A T I O N   N O T E   ************************************
*
* Title			: DMX512 reception library
* Version		: v1.2
* Last updated	: 21.04.07
* Target		: Transceiver Rev.3.01 [ATmega8515]
* Clock			: 8MHz, 16MHz
*
* written by hendrik hoelscher, www.hoelscher-hi.de
***************************************************************************
 This program is free software; you can redistribute it and/or 
 modify it under the terms of the GNU General Public License 
 as published by the Free Software Foundation; either version2 of 
 the License, or (at your option) any later version. 

 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 General Public License for more details. 

 If you have no copy of the GNU General Public License, write to the 
 Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

 For other license models, please contact the author.

;***************************************************************************/

#include "lib_dmx_in.h"
#include "globals.h"
#include <avr/wdt.h>

// ********************* local definitions *********************

enum {IDLE, BREAK, STARTB, STARTADR, DATA, ERR};			//DMX states

		 uint8_t 	 gDmxState;
		 uint8_t 	*gDmxPnt;
		 uint16_t	 DmxCount;
		 uint16_t	 TimeBuf;



// *************** DMX Reception Initialisation ****************
void init_DMX_RX(void)
{
DDRD  |= (1<<2);
PORTD &= ~(1<<2);								//enable reception
UBRRH  = 0;
UBRRL  = ((F_OSC/4000)-1);						//250kbaud, 8N2
UCSRC  = (1<<URSEL)|(3<<UCSZ0)|(1<<USBS);
UCSRB  = (1<<RXEN)|(1<<RXCIE);
DmxAddress= 1;
gDmxState= IDLE;
DmxCount= 0;

uint8_t i;
for (i=0; i<sizeof(DmxRxField); i++)
	{
	DmxRxField[i]= 0;
	}
}




uint8_t check_signal(void)						//return 0 if a byte arrives within 0.1s
{
init_DMX_RX();
uint8_t res= 1;
uint8_t cnt= 5;
wdt_reset();
Flags &= ~(1<<TICK);
Flags &= ~(1<<SIGNAL);
while (cnt != 0)
	{
	if (Flags &(1<<TICK))						//wait 0.1s
		{
		Flags &= ~(1<<TICK);
		cnt--;
		}
	if (Flags &(1<<SIGNAL)) res= 0;
	wdt_reset();
	}
return(res);
}


void get_DmxLen(void)							//get break and MAB length
{
UCSRB  = 0;
cli();
wdt_reset();
loop_until_bit_is_set(PIND, PD0);
do {
	loop_until_bit_is_clear(PIND, PD0);			//falling edge
	TCNT1= 0;
	loop_until_bit_is_set(PIND, PD0);			//rising edge -> end of break
	}
while (TCNT1 < 80);
BreakLen= TCNT1 -1;
loop_until_bit_is_clear(PIND, PD0);				//falling edge -> end of MAB
MabLen= TCNT1 -BreakLen;
sei();
UCSRB  = (1<<RXEN)|(1<<RXCIE);
gDmxState= IDLE;
DmxCount= 0;
}


// *************** DMX Reception ISR ****************
ISR (UART_RX_vect)
{
uint8_t USARTstate= UCSRA;						//get state
uint8_t DmxByte= UDR;							//get data
uint8_t DmxState= gDmxState;					//just get once from SRAM!!!
 
if (USARTstate &(1<<FE))						//check for break
	{
	UCSRA &= ~(1<<FE);							//reset flag
	gDmxState= BREAK;
	gDmxPnt= ((uint8_t*)DmxRxField +2);
	return;
	}

if (DmxState != ERR) DmxCount++;				//just count DMX bytes

if (DmxState == BREAK)
	{
	DmxSize= DmxCount -1;
	if (DmxByte == 0) 
		{
		Flags |= (1<<SIGNAL);					//mark: we have DMX!
		gDmxState= STARTB;						//normal start code detected
		DmxRefresh++;
		DmxCount=  0;							//reset frame counter
		}
	else if (DmxByte == 0xCC)
		{										//RDM sc detected
		RdmRefresh++;
		gDmxState= ERR;
		}
	else
		{
		ErrRefresh++;							//invalid sc
		gDmxState= ERR;
		}
	}
	
else if (DmxState == STARTB)
	{
	if (DmxCount == DmxAddress)					//start address reached?
		{
		TimeBuf= TCNT1;
		gDmxState= STARTADR;
		DmxRxField[0]= DmxByte;
		}
	}

else if (DmxState == STARTADR)
	{
	IbgLen= (TCNT1 -TimeBuf) -43;				//calc interbyte gap
	DmxRxField[1]= DmxByte;
	gDmxState= DATA;
	}

else if (DmxState == DATA)
	{
	uint8_t *DmxPnt;
	DmxPnt= gDmxPnt;
	*DmxPnt= DmxByte;
	if (++DmxPnt >= (DmxRxField +sizeof(DmxRxField))) 	//all ch received?
		{
		gDmxState= IDLE;
		}
	else gDmxPnt= DmxPnt;
	}							
}

